@@ -1,4 +1,4 @@
-__version__ = '0.13.17'
+__version__ = '0.13.20'
 
 import os
 
@@ -6,7 +6,7 @@ from .document import Document
 from .array import DocumentArray
 from .dataclasses import dataclass, field
 
-if 'DA_NO_RICH_HANDLER' not in os.environ:
+if 'DA_RICH_HANDLER' in os.environ:
     from rich.traceback import install
 
     install()
