@@ -1,11 +1,8 @@
-import itertools
 import uuid
 from dataclasses import dataclass, field
 from typing import (
-    Generator,
-    Iterator,
+    Iterable,
     Dict,
-    Sequence,
     Optional,
     TYPE_CHECKING,
     Union,
@@ -15,15 +12,14 @@ from typing import (
 
 import numpy as np
 import weaviate
-from weaviate.auth import AuthCredentials
 
-from ..base.backend import BaseBackendMixin
 from .... import Document
-from ....helper import dataclass_from_dict
+from ....helper import dataclass_from_dict, filter_dict, _safe_cast_int
+from ..base.backend import BaseBackendMixin, TypeMap
 from ..registry import _REGISTRY
 
 if TYPE_CHECKING:
-    from ....types import DocumentArraySourceType, ArrayType
+    from ....typing import ArrayType, DocumentArraySourceType
 
 
 @dataclass
@@ -37,11 +33,31 @@ class WeaviateConfig:
     name: Optional[str] = None
     serialize_config: Dict = field(default_factory=dict)
     n_dim: Optional[int] = None  # deprecated, not used anymore since weaviate 1.10
+    # vectorIndexConfig parameters
+    ef: Optional[int] = None
+    ef_construction: Optional[int] = None
+    timeout_config: Optional[Tuple[int, int]] = None
+    max_connections: Optional[int] = None
+    dynamic_ef_min: Optional[int] = None
+    dynamic_ef_max: Optional[int] = None
+    dynamic_ef_factor: Optional[int] = None
+    vector_cache_max_objects: Optional[int] = None
+    flat_search_cutoff: Optional[int] = None
+    cleanup_interval_seconds: Optional[int] = None
+    skip: Optional[bool] = None
+    columns: Optional[List[Tuple[str, str]]] = None
+    distance: Optional[str] = None
 
 
 class BackendMixin(BaseBackendMixin):
     """Provide necessary functions to enable this storage backend."""
 
+    TYPE_MAP = {
+        'str': TypeMap(type='string', converter=str),
+        'float': TypeMap(type='number', converter=float),
+        'int': TypeMap(type='int', converter=_safe_cast_int),
+    }
+
     def _init_storage(
         self,
         _docs: Optional['DocumentArraySourceType'] = None,
@@ -62,8 +78,6 @@ class BackendMixin(BaseBackendMixin):
         elif isinstance(config, dict):
             config = dataclass_from_dict(WeaviateConfig, config)
 
-        from ... import DocumentArray
-
         self._serialize_config = config.serialize_config
 
         if config.name and config.name != config.name.capitalize():
@@ -75,10 +89,14 @@ class BackendMixin(BaseBackendMixin):
         self._persist = bool(config.name)
 
         self._client = weaviate.Client(
-            f'{config.protocol}://{config.host}:{config.port}'
+            f'{config.protocol}://{config.host}:{config.port}',
+            timeout_config=config.timeout_config,
         )
         self._config = config
 
+        if self._config.columns is None:
+            self._config.columns = []
+
         self._schemas = self._load_or_create_weaviate_schema()
 
         _REGISTRY[self.__class__.__name__][self._class_name].append(self)
@@ -90,9 +108,7 @@ class BackendMixin(BaseBackendMixin):
         # table and load the given `docs`
         if _docs is None:
             return
-        elif isinstance(
-            _docs, (DocumentArray, Sequence, Generator, Iterator, itertools.chain)
-        ):
+        elif isinstance(_docs, Iterable):
             self.clear()
             self.extend(_docs)
         else:
@@ -107,7 +123,7 @@ class BackendMixin(BaseBackendMixin):
         :return: string representing the name of  weaviate class/schema name of
             this :class:`DocumentArrayWeaviate` object
         """
-        return ''.join([i for i in uuid.uuid1().hex if not i.isdigit()]).capitalize()
+        return f'Class{uuid.uuid4().hex}'
 
     def _get_schema_by_name(self, cls_name: str) -> Dict:
         """Return the schema dictionary object with the class name
@@ -119,12 +135,26 @@ class BackendMixin(BaseBackendMixin):
         """
         # TODO: ideally we should only use one schema. this will allow us to deal with
         # consistency better
-        return {
+        hnsw_config = {
+            'ef': self._config.ef,
+            'efConstruction': self._config.ef_construction,
+            'maxConnections': self._config.max_connections,
+            'dynamicEfMin': self._config.dynamic_ef_min,
+            'dynamicEfMax': self._config.dynamic_ef_max,
+            'dynamicEfFactor': self._config.dynamic_ef_factor,
+            'vectorCacheMaxObjects': self._config.vector_cache_max_objects,
+            'flatSearchCutoff': self._config.flat_search_cutoff,
+            'cleanupIntervalSeconds': self._config.cleanup_interval_seconds,
+            'skip': self._config.skip,
+            'distance': self._config.distance,
+        }
+
+        base_classes = {
             'classes': [
                 {
                     'class': cls_name,
                     "vectorizer": "none",
-                    'vectorIndexConfig': {'skip': False},
+                    'vectorIndexConfig': {'skip': False, **filter_dict(hnsw_config)},
                     'properties': [
                         {
                             'dataType': ['blob'],
@@ -147,6 +177,15 @@ class BackendMixin(BaseBackendMixin):
                 },
             ]
         }
+        for col, coltype in self._config.columns:
+            new_property = {
+                'dataType': [self._map_type(coltype)],
+                'name': col,
+                'indexInverted': True,
+            }
+            base_classes['classes'][0]['properties'].append(new_property)
+
+        return base_classes
 
     def _load_or_create_weaviate_schema(self):
         """Create a new weaviate schema for this :class:`DocumentArrayWeaviate` object
@@ -275,8 +314,17 @@ class BackendMixin(BaseBackendMixin):
         :param value: document to create a payload for
         :return: the payload dictionary
         """
+        columns_dict = {key: val for [key, val] in self._config.columns}
+        extra_columns = {
+            col: self._map_column(value.tags.get(col), columns_dict[col])
+            for col, _ in self._config.columns
+        }
+
         return dict(
-            data_object={'_serialized': value.to_base64(**self._serialize_config)},
+            data_object={
+                '_serialized': value.to_base64(**self._serialize_config),
+                **extra_columns,
+            },
             class_name=self._class_name,
             uuid=self._map_id(value.id),
             vector=self._map_embedding(value.embedding),
@@ -312,16 +360,6 @@ class BackendMixin(BaseBackendMixin):
             embedding = None
         return embedding
 
-    def _get_storage_infos(self) -> Dict:
-        return {
-            'Backend': 'Weaviate',
-            'Host': self._config.host,
-            'Port': str(self._config.port),
-            'Protocol': self._config.protocol,
-            'Schema Name': self._config.name,
-            'Serialization Protocol': self._config.serialize_config.get('protocol'),
-        }
-
     def __getstate__(self):
         d = dict(self.__dict__)
         del d['_client']
